const db = require("../db");
const authwrap = require("./authwrap");

const create_thread = async (msg, respond, socket, io) => {
  // validate inputs
  if (!msg.name) {
    return respond({
      success: false,
      message: "thread name required",
    });
  }
  if (msg.name.length > 200) {
    return respond({
      success: false,
      message: "thread name 200 chars max",
    });
  }
  // add to db
  const insert = await db.query(
    "insert into threads (name, creator) values (?, ?) returning id",
    [msg.name, msg.auth_user.id]
  );
  const thread_id = insert.rows[0].id;
  // set up permissions
  if (!msg.permissions || !msg.permissions.view_limited) {
    await db.query(
      `insert into permissions (thread, type, flexible, permission, value) 
      values (?, ?, ?, ?, ?)`,
      [thread_id, "everyone", false, "view", "true"]
    );
    if (!msg.permissions || !msg.permissions.post_limited) {
      await db.query(
        `insert into permissions (thread, type, flexible, permission, value) 
        values (?, ?, ?, ?, ?)`,
        [thread_id, "everyone", false, "post", "true"]
      );
    } else {
      await db.query(
        `insert into permissions (thread, type, flexible, permission, value) 
        values (?, ?, ?, ?, ?)`,
        [thread_id, "members", false, "post", "true"]
      );
    }
  } else {
    await db.query(
      `insert into permissions (thread, type, flexible, permission, value) 
      values (?, ?, ?, ?, ?)`,
      [thread_id, "members", false, "view", "true"]
    );
    await db.query(
      `insert into permissions (thread, type, flexible, permission, value) 
      values (?, ?, ?, ?, ?)`,
      [thread_id, "members", false, "post", "true"]
    );
  }
  // add members
  for (let user of msg.members) {
    // get user id
    const id = await db.query("select id from users where name = ?", [
      user.user,
    ]);
    if (id.rows.length > 0) {
      const user_id = id.rows[0].id;
      await db.query(
        "insert into members (thread, user, key_delivery) values (?, ?, ?)",
        [thread_id, user_id, user.key]
      );
    }
  }
  const member_perms = {
    is_member: true,
    view: true,
    post: true,
  };
  const general_perms = {
    is_member: false,
    view: !msg.permissions || !msg.permissions.view_limited,
    post: !msg.permissions || !msg.permissions.post_limited,
  };
  for (let username in io.cache) {
    let member = msg.members.filter((i) => i.user === username)[0];
    if (member) {
      const sockets = io.cache[username];
      for (let s of sockets) {
        io.to(s).emit("new_thread", {
          name: msg.name,
          id: insert.rows[0].id,
          permissions: member_perms,
          key: member.key,
        });
      }
    } else if (general_perms.view) {
      const sockets = io.cache[username];
      for (let s of sockets) {
        io.to(s).emit("new_thread", {
          name: msg.name,
          id: insert.rows[0].id,
          permissions: general_perms,
        });
      }
    }
  }
  // respond
  return respond({
    success: true,
    id: insert.rows[0].id,
  });
};

module.exports = authwrap(create_thread);
