const db = require("../db");
const authwrap = require("./authwrap");
const check_permissions = require("./helpers/check_permissions");

const send_message = async (msg, respond, socket, io) => {
  if (!msg.thread) {
    return respond({
      success: false,
      message: "thread ID required",
    });
  }
  if (!(await check_permissions(msg.auth_user.id, msg.thread)).post) {
    return respond({
      success: false,
      message: "you can't post to this thread",
    });
  }
  // add message and send it to everyone
  const id = await db.query(
    "insert into posts (user, thread, content, encrypted) values (?, ?, ?, ?) returning id",
    [msg.auth_user.id, msg.thread, msg.message, msg.encrypted]
  );
  // get thread members
  const members = (
    await db.query(
      "select name from users join members on members.user = users.id where members.thread = ?",
      [msg.thread]
    )
  ).rows.map((i) => i.name);
  // get perms
  const permissions = await db.query(
    "select * from permissions where thread = ? and type = 'everyone' and value = 'true' and permission = 'view'",
    [msg.thread]
  );
  for (let username in io.cache) {
    if (members.includes(username)) {
      const sockets = io.cache[username];
      for (let s of sockets) {
        io.to(s).emit("new_message", {
          id: id.rows[0].id,
          name: msg.auth_user.name,
          message: msg.message,
          thread: msg.thread,
        });
      }
    } else if (permissions.rows.length > 0) {
      const sockets = io.cache[username];
      for (let s of sockets) {
        io.to(s).emit("new_message", {
          id: id.rows[0].id,
          name: msg.auth_user.name,
          message: msg.message,
          thread: msg.thread,
        });
      }
    }
  }
  return respond({
    success: true,
    id: id.rows[0].id,
  });
};

module.exports = authwrap(send_message);
